const express = require('express');
const { Cruds } = require('./cruds');
const { getModelName, createSchema } = require('./helper');
const { validationMiddleware } = require('./validation');

const router = express.Router();

exports.default = (model, {
  path = `/${getModelName(model)}`,
  prefix = '',
  skipDelete = false,
  skipAdd = false,
  skipUpdate = false,
  skipFind = false,
  skipGet = false,
}) => {
  const modelService = new Cruds(model);
  const schema = model.tableAttributes;
  const findVs = createSchema(schema, 'find');
  const addVs = createSchema(schema, 'add');
  const updateVs = createSchema(schema, 'update');
  if (!skipFind) {
    router.get(prefix + path, validationMiddleware(findVs, true), async (req, res, next) => {
      try {
        const params = req.query || {};
        const records = await modelService.findAll(params);
        return res.json(records);
      } catch (error) {
        return next(error);
      }
    });
  }
  if (!skipAdd) {
    router.post(prefix + path, validationMiddleware(addVs), async (req, res, next) => {
      try {
        const params = req.body || {};
        const record = await modelService.add(params);
        return res.json(record);
      } catch (error) {
        return next(error);
      }
    });
  }
  if (!skipUpdate) {
    router.put(`${prefix + path}/:id`, validationMiddleware(updateVs), async (req, res, next) => {
      try {
        const params = req.body || {};
        const { id } = req.params;
        const record = await modelService.updateById(id, params);
        return res.json(record);
      } catch (error) {
        return next(error);
      }
    });
  }
  if (!skipGet) {
    router.get(`${prefix + path}/:id`, async (req, res, next) => {
      try {
        const { id } = req.params;
        const record = await modelService.findById(id);
        if (record) {
          return res.json(record);
        }
        return next(new Error('Object not found'));
      } catch (error) {
        return next(error);
      }
    });
  }
  if (!skipDelete) {
    router.delete(`${prefix + path}/:id`, async (req, res, next) => {
      try {
        const { id } = req.params;
        const record = await modelService.delete(id);
        return res.json(record);
      } catch (error) {
        return next(error);
      }
    });
  }
  return router;
};
